#  Copyright (c) 1997-2011
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
#  http://www.polymake.de
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-------------------------------------------------------------------------------
#  $Project: polymake $$Id: extension.make 10162 2011-04-21 15:51:29Z benmuell $


###########################################################
#
#  Building and installing a polymake extension
#

ExtensionName := $(notdir ${CURDIR})

.PHONY: all compile install install-arch install-shared clean TAGS RTAGS test

include ${ProjectTop}/support/utils.make
ifndef BuildDir
  BuildDir := $(call _LocateBuildDir)
  ifndef BuildDir
    $(error CONFIGURATION ERROR)
  endif
endif
include ${BuildDir}/conf.make

### default target
all : compile

InstallTop  := ${InstallTop}/extensions/${ExtensionName}
InstallArch := ${InstallArch}/extensions/${ExtensionName}

Apps := $(notdir $(wildcard ${BuildDir}/apps/*))

define _MakeApp
	$(MAKE) -C $(notdir ${BuildDir})/apps/$(1) AppName=$(1) $(2)

endef
define _MakeApps
	$(foreach a,${Apps},$(call _MakeApp,$a,$(1)))
endef
define _InstallSubdir
${PERL} ${INSTALL_PL} -m ${DirMask} -U $(2) $(1) ${InstallTop}/$(1)

endef
define _InstallShared
	$(foreach d, perllib rules scripts, $(if $(wildcard $(1)/$d), $(call _InstallSubdir,$(1)/$d)))
endef
define _CreateDir
${PERL} ${INSTALL_PL} -d -m ${DirMask} $(1)

endef

compile clean:
	@+$(call _MakeApps, $@)

install : install-shared install-arch

install-arch : compile
	$(call _CreateDir,${InstallArch}/lib)
	@+$(call _MakeApps, install InstallTop=${InstallTop} InstallArch=${InstallArch})
	${PERL} ${INSTALL_PL} -m 444 ${BuildDir}/conf.make ${InstallArch}

install-shared:
	$(call _CreateDir,${InstallTop})
	$(foreach a, ${Apps}, $(call _InstallShared,apps/$a))
	@+$(call _MakeApps, install-src InstallTop=${InstallTop})
	$(if $(wildcard scripts), $(call _InstallSubdir,scripts))
	if [ -f URI ]; then \
	  ${PERL} ${INSTALL_PL} -m 444 URI ${InstallTop}; \
	fi

distclean:
	rm -rf ${BuildDir}

tagsFLAGS = -R -e -f $@ --exclude=.svn --exclude='.\#*' --exclude='\#*' --exclude='*~'

TAGS:
	ctags ${tagsFLAGS} --language-force=c++ apps/*/{src,include}
RTAGS:
	ctags ${tagsFLAGS} --language-force=perl $(wildcard apps/*/{perllib,rules,scripts} scripts)

AppsWithTests := $(foreach a,${Apps},$(if $(wildcard apps/$a/testsuite), $a))

ifdef AppsWithTests
test : all
	${PERL} $(firstword $(wildcard ${ProjectTop}/perl/polymake ${InstallBin}/polymake)) --script run_testcases --applications ${AppsWithTests} --extensions ${CURDIR}
else
test :
	@echo No testcases found!
	@false
endif
