<?xml version="1.0"?>

<!--
  Copyright (c) 1997-2011
  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
  http://www.polymake.de

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version: http://www.gnu.org/licenses/gpl.txt.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
===============================================================================
  $Project: polymake $$Id: upgrade-2.10.xslt 10290 2011-06-09 16:51:02Z herr $

  
  upgrade-2.10.xslt:
  The second part of the upgrade script for Release 2.10.
  FIXME: MERGE upgrade-2.9.16.xslt into upgrade-2.10.xslt!!! 
-->

<xsl:transform version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
			     xmlns:p="http://www.math.tu-berlin.de/polymake/#3">

<xsl:output method="xml" encoding="utf-8" cdata-section-elements="p:description" indent="yes" />


<!-- upgrade-2.9.15 -->

<!-- We search for objects with property POLYTOPAL_SUBDIVISION -->

<xsl:template match="p:object">
   <xsl:variable name="has_POLYTOPAL_SUBDIVISION" select="./p:property[@name='POLYTOPAL_SUBDIVISION']" />
   <xsl:variable name="has_TRIANGULATION" select="./p:property[@name='TRIANGULATION']" />
   <xsl:variable name="has_WEIGHTS" select="./p:property[@name='WEIGHTS']" />
   <xsl:variable name="has_WEIGHTS_GENERIC" select="./p:property[@name='WEIGHTS_GENERIC']" />
   <xsl:variable name='weights_generic'>
     <xsl:choose>
       <xsl:when test="(count($has_WEIGHTS_GENERIC)>=1)">
         <xsl:value-of select="./p:property[@name='WEIGHTS_GENERIC']/@value" />
       </xsl:when>
       <xsl:otherwise>
         <xsl:text>false</xsl:text>
       </xsl:otherwise>
     </xsl:choose>
    </xsl:variable>
    <xsl:element name="object" namespace="{namespace-uri()}">
      <xsl:copy-of select="@*[name()!='tm' and name()!='version']" />
      <xsl:choose>

        <xsl:when test="(count($has_POLYTOPAL_SUBDIVISION)>=1)or ((count($has_WEIGHTS)>=1) and (starts-with($weights_generic,'false')))">
            <!-- we have a polytopal subdivision-->

	    <!-- most properties stay in the object -->
	
	    <xsl:for-each select="*">
              <xsl:variable name="PD-props">
                <xsl:apply-templates mode="PD-props" select="." />
              </xsl:variable>
              <xsl:if test="($PD-props='')">
                <xsl:apply-templates select="." />
              </xsl:if>   
	    </xsl:for-each>
	    
	    <!-- copy some properties into the polytopal-subdivision object -->
	    	    
	    <xsl:element name="property" namespace="{namespace-uri()}">
              <xsl:attribute name="name">fan::POLYTOPAL_SUBDIVISION</xsl:attribute>
              <xsl:element name="object" namespace="{namespace-uri()}">
                <xsl:for-each select="*">
                  <xsl:variable name="PD-props">
                    <xsl:apply-templates mode="PD-props" select="." />
                  </xsl:variable>
                  <xsl:if test="($PD-props!='')">
                    <xsl:element name="property" namespace="{namespace-uri()}">
                      <xsl:attribute name="name">
                        <xsl:value-of select="$PD-props" />
                      </xsl:attribute>
                      <xsl:copy-of select="@*[name()!='name']" />
                      <xsl:apply-templates select="./node()" />
                    </xsl:element>
                  </xsl:if>
                </xsl:for-each>
             </xsl:element>
           </xsl:element>	    

        </xsl:when>

        <xsl:when test="($weights_generic='true') and (count($has_WEIGHTS)>=1)">
          <!--our weight vector defines a triangulation-->
          <xsl:choose>
            <xsl:when test="(count($has_TRIANGULATION)>=1)">
              <!-- if there is a triangulation, we just copy WEIGHTS into it-->
              <xsl:apply-templates  mode='wg' />
            </xsl:when>
            <xsl:otherwise>
              <!-- otherwise we have to create the property-->
        
              <!-- first we just copy all other properties -->
              <xsl:for-each select="*">
                <xsl:apply-templates select="." />
	      </xsl:for-each>
	    
	    <!-- then we create the triangulation object -->
	    	    
	    <xsl:element name="property" namespace="{namespace-uri()}">
              <xsl:attribute name="name">TRIANGULATION</xsl:attribute>
              <xsl:element name="object" namespace="{namespace-uri()}">
                <xsl:element name="property" namespace="{namespace-uri()}">
                  <xsl:attribute name="name">WEIGHTS</xsl:attribute>
                  <xsl:element name="v" namespace="{namespace-uri()}">
                    <xsl:value-of select="p:property[@name='WEIGHTS']/p:v/text()"/>
                  </xsl:element>
                </xsl:element>
              </xsl:element>
            </xsl:element>
           
        </xsl:otherwise>
      </xsl:choose>

        </xsl:when>


        <xsl:otherwise>
          <xsl:apply-templates  />
        </xsl:otherwise>


     </xsl:choose>
  </xsl:element>
</xsl:template>



<!-- These properties will be moved into POLYTOPAL_SUBDIVISION  -->

<xsl:template mode="PD-props" match="p:property[@name='SPLITS_IN_SUBDIVISION']">
   <xsl:text>fan::REFINED_SPLITS</xsl:text>
</xsl:template>

<xsl:template mode="PD-props" match="p:property[@name='WEIGHTS']">
    <xsl:text>fan::WEIGHTS</xsl:text>
</xsl:template>

<xsl:template mode="PD-props" match="p:property[@name='POLYTOPAL_SUBDIVISION']">
    <xsl:text>MAXIMAL_CELLS</xsl:text>
</xsl:template>

<xsl:template mode="PD-props" match="*" />


<!-- get WEIGHTS into TRIANGULATION -->

<xsl:template mode='wg' match="p:property[@name='TRIANGULATION']">
  <xsl:element name="property" namespace="{namespace-uri()}">
    <xsl:attribute name="name">TRIANGULATION</xsl:attribute>
    <xsl:element name="object" namespace="{namespace-uri()}">
      <xsl:element name="property" namespace="{namespace-uri()}">
        <xsl:attribute name="name">WEIGHTS</xsl:attribute>
        <xsl:element name="v" namespace="{namespace-uri()}">
          <xsl:value-of select="../p:property[@name='WEIGHTS']/p:v/text()"/>
        </xsl:element>
      </xsl:element>
  <xsl:apply-templates select="./p:object/node()" />
    </xsl:element>
  </xsl:element>      
</xsl:template>

<xsl:template match="*" mode="wg">
  <xsl:apply-templates select="." />
</xsl:template>

<!-- remove properties WEIGHTS and WEIGHTS_GENERIC -->

<xsl:template match="p:property[@name='WEIGHTS_GENERIC']">
</xsl:template>

<xsl:template match="p:property[@name='WEIGHTS']">
</xsl:template>



<xsl:include href="trivial-copy.xslt" />


</xsl:transform>
